@@ -18,16 +18,20 @@
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
 """This module contains the BasePersistence class."""
 import warnings
+from sys import version_info as py_ver
 from abc import ABC, abstractmethod
 from copy import copy
-from typing import Any, DefaultDict, Dict, Optional, Tuple, cast, ClassVar
+from typing import Dict, Optional, Tuple, cast, ClassVar, Generic, DefaultDict
+
+from telegram.utils.deprecate import set_new_attribute_deprecated
 
 from telegram import Bot
+import telegram.ext.extbot
 
-from telegram.utils.types import ConversationDict
+from telegram.ext.utils.types import UD, CD, BD, ConversationDict, CDCData
 
 
-class BasePersistence(ABC):
+class BasePersistence(Generic[UD, CD, BD], ABC):
     """Interface class for adding persistence to your bot.
     Subclass this object for different implementations of a persistent bot.
 
@@ -35,16 +39,22 @@ class BasePersistence(ABC):
 
     * :meth:`get_bot_data`
     * :meth:`update_bot_data`
+    * :meth:`refresh_bot_data`
     * :meth:`get_chat_data`
     * :meth:`update_chat_data`
+    * :meth:`refresh_chat_data`
     * :meth:`get_user_data`
     * :meth:`update_user_data`
+    * :meth:`refresh_user_data`
+    * :meth:`get_callback_data`
+    * :meth:`update_callback_data`
     * :meth:`get_conversations`
     * :meth:`update_conversation`
     * :meth:`flush`
 
     If you don't actually need one of those methods, a simple ``pass`` is enough. For example, if
-    ``store_bot_data=False``, you don't need :meth:`get_bot_data` and :meth:`update_bot_data`.
+    ``store_bot_data=False``, you don't need :meth:`get_bot_data`, :meth:`update_bot_data` or
+    :meth:`refresh_bot_data`.
 
     Warning:
         Persistence will try to replace :class:`telegram.Bot` instances by :attr:`REPLACED_BOT` and
@@ -64,7 +74,11 @@ class BasePersistence(ABC):
         store_chat_data (:obj:`bool`, optional): Whether chat_data should be saved by this
             persistence class. Default is :obj:`True` .
         store_bot_data (:obj:`bool`, optional): Whether bot_data should be saved by this
-            persistence class. Default is :obj:`True` .
+            persistence class. Default is :obj:`True`.
+        store_callback_data (:obj:`bool`, optional): Whether callback_data should be saved by this
+            persistence class. Default is :obj:`False`.
+
+            .. versionadded:: 13.6
 
     Attributes:
         store_user_data (:obj:`bool`): Optional, Whether user_data should be saved by this
@@ -73,41 +87,88 @@ class BasePersistence(ABC):
             persistence class.
         store_bot_data (:obj:`bool`): Optional. Whether bot_data should be saved by this
             persistence class.
+        store_callback_data (:obj:`bool`): Optional. Whether callback_data should be saved by this
+            persistence class.
+
+            .. versionadded:: 13.6
     """
 
-    def __new__(cls, *args: Any, **kwargs: Any) -> 'BasePersistence':  # pylint: disable=W0613
+    # Apparently Py 3.7 and below have '__dict__' in ABC
+    if py_ver < (3, 7):
+        __slots__ = (
+            'store_user_data',
+            'store_chat_data',
+            'store_bot_data',
+            'store_callback_data',
+            'bot',
+        )
+    else:
+        __slots__ = (
+            'store_user_data',  # type: ignore[assignment]
+            'store_chat_data',
+            'store_bot_data',
+            'store_callback_data',
+            'bot',
+            '__dict__',
+        )
+
+    def __new__(
+        cls, *args: object, **kwargs: object  # pylint: disable=W0613
+    ) -> 'BasePersistence':
+        """This overrides the get_* and update_* methods to use insert/replace_bot.
+        That has the side effect that we always pass deepcopied data to those methods, so in
+        Pickle/DictPersistence we don't have to worry about copying the data again.
+
+        Note: This doesn't hold for second tuple-entry of callback_data. That's a Dict[str, str],
+        so no bots to replace anyway.
+        """
         instance = super().__new__(cls)
         get_user_data = instance.get_user_data
         get_chat_data = instance.get_chat_data
         get_bot_data = instance.get_bot_data
+        get_callback_data = instance.get_callback_data
         update_user_data = instance.update_user_data
         update_chat_data = instance.update_chat_data
         update_bot_data = instance.update_bot_data
+        update_callback_data = instance.update_callback_data
 
-        def get_user_data_insert_bot() -> DefaultDict[int, Dict[Any, Any]]:
+        def get_user_data_insert_bot() -> DefaultDict[int, UD]:
             return instance.insert_bot(get_user_data())
 
-        def get_chat_data_insert_bot() -> DefaultDict[int, Dict[Any, Any]]:
+        def get_chat_data_insert_bot() -> DefaultDict[int, CD]:
             return instance.insert_bot(get_chat_data())
 
-        def get_bot_data_insert_bot() -> Dict[Any, Any]:
+        def get_bot_data_insert_bot() -> BD:
             return instance.insert_bot(get_bot_data())
 
-        def update_user_data_replace_bot(user_id: int, data: Dict) -> None:
+        def get_callback_data_insert_bot() -> Optional[CDCData]:
+            cdc_data = get_callback_data()
+            if cdc_data is None:
+                return None
+            return instance.insert_bot(cdc_data[0]), cdc_data[1]
+
+        def update_user_data_replace_bot(user_id: int, data: UD) -> None:
             return update_user_data(user_id, instance.replace_bot(data))
 
-        def update_chat_data_replace_bot(chat_id: int, data: Dict) -> None:
+        def update_chat_data_replace_bot(chat_id: int, data: CD) -> None:
             return update_chat_data(chat_id, instance.replace_bot(data))
 
-        def update_bot_data_replace_bot(data: Dict) -> None:
+        def update_bot_data_replace_bot(data: BD) -> None:
             return update_bot_data(instance.replace_bot(data))
 
-        instance.get_user_data = get_user_data_insert_bot
-        instance.get_chat_data = get_chat_data_insert_bot
-        instance.get_bot_data = get_bot_data_insert_bot
-        instance.update_user_data = update_user_data_replace_bot
-        instance.update_chat_data = update_chat_data_replace_bot
-        instance.update_bot_data = update_bot_data_replace_bot
+        def update_callback_data_replace_bot(data: CDCData) -> None:
+            obj_data, queue = data
+            return update_callback_data((instance.replace_bot(obj_data), queue))
+
+        # We want to ignore TGDeprecation warnings so we use obj.__setattr__. Adds to __dict__
+        object.__setattr__(instance, 'get_user_data', get_user_data_insert_bot)
+        object.__setattr__(instance, 'get_chat_data', get_chat_data_insert_bot)
+        object.__setattr__(instance, 'get_bot_data', get_bot_data_insert_bot)
+        object.__setattr__(instance, 'get_callback_data', get_callback_data_insert_bot)
+        object.__setattr__(instance, 'update_user_data', update_user_data_replace_bot)
+        object.__setattr__(instance, 'update_chat_data', update_chat_data_replace_bot)
+        object.__setattr__(instance, 'update_bot_data', update_bot_data_replace_bot)
+        object.__setattr__(instance, 'update_callback_data', update_callback_data_replace_bot)
         return instance
 
     def __init__(
@@ -115,18 +176,33 @@ class BasePersistence(ABC):
         store_user_data: bool = True,
         store_chat_data: bool = True,
         store_bot_data: bool = True,
+        store_callback_data: bool = False,
     ):
         self.store_user_data = store_user_data
         self.store_chat_data = store_chat_data
         self.store_bot_data = store_bot_data
+        self.store_callback_data = store_callback_data
         self.bot: Bot = None  # type: ignore[assignment]
 
+    def __setattr__(self, key: str, value: object) -> None:
+        # Allow user defined subclasses to have custom attributes.
+        if issubclass(self.__class__, BasePersistence) and self.__class__.__name__ not in {
+            'DictPersistence',
+            'PicklePersistence',
+        }:
+            object.__setattr__(self, key, value)
+            return
+        set_new_attribute_deprecated(self, key, value)
+
     def set_bot(self, bot: Bot) -> None:
         """Set the Bot to be used by this persistence instance.
 
         Args:
             bot (:class:`telegram.Bot`): The bot.
         """
+        if self.store_callback_data and not isinstance(bot, telegram.ext.extbot.ExtBot):
+            raise TypeError('store_callback_data can only be used with telegram.ext.ExtBot.')
+
         self.bot = bot
 
     @classmethod
@@ -135,7 +211,9 @@ class BasePersistence(ABC):
         Replaces all instances of :class:`telegram.Bot` that occur within the passed object with
         :attr:`REPLACED_BOT`. Currently, this handles objects of type ``list``, ``tuple``, ``set``,
         ``frozenset``, ``dict``, ``defaultdict`` and objects that have a ``__dict__`` or
-        ``__slot__`` attribute, excluding objects that can't be copied with `copy.copy`.
+        ``__slots__`` attribute, excluding classes and objects that can't be copied with
+        ``copy.copy``. If the parsing of an object fails, the object will be returned unchanged and
+        the error will be logged.
 
         Args:
             obj (:obj:`object`): The object
@@ -146,7 +224,7 @@ class BasePersistence(ABC):
         return cls._replace_bot(obj, {})
 
     @classmethod
-    def _replace_bot(cls, obj: object, memo: Dict[int, Any]) -> object:  # pylint: disable=R0911
+    def _replace_bot(cls, obj: object, memo: Dict[int, object]) -> object:  # pylint: disable=R0911
         obj_id = id(obj)
         if obj_id in memo:
             return memo[obj_id]
@@ -166,6 +244,14 @@ class BasePersistence(ABC):
             new_immutable = obj.__class__(cls._replace_bot(item, memo) for item in obj)
             memo[obj_id] = new_immutable
             return new_immutable
+        if isinstance(obj, type):
+            # classes usually do have a __dict__, but it's not writable
+            warnings.warn(
+                'BasePersistence.replace_bot does not handle classes. See '
+                'the docs of BasePersistence.replace_bot for more information.',
+                RuntimeWarning,
+            )
+            return obj
 
         try:
             new_obj = copy(obj)
@@ -191,21 +277,34 @@ class BasePersistence(ABC):
                 new_obj[cls._replace_bot(k, memo)] = cls._replace_bot(val, memo)
             memo[obj_id] = new_obj
             return new_obj
-        if hasattr(obj, '__dict__'):
-            for attr_name, attr in new_obj.__dict__.items():
-                setattr(new_obj, attr_name, cls._replace_bot(attr, memo))
-            memo[obj_id] = new_obj
-            return new_obj
-        if hasattr(obj, '__slots__'):
-            for attr_name in new_obj.__slots__:
-                setattr(
-                    new_obj,
-                    attr_name,
-                    cls._replace_bot(cls._replace_bot(getattr(new_obj, attr_name), memo), memo),
-                )
-            memo[obj_id] = new_obj
-            return new_obj
+        try:
+            if hasattr(obj, '__slots__'):
+                for attr_name in new_obj.__slots__:
+                    setattr(
+                        new_obj,
+                        attr_name,
+                        cls._replace_bot(
+                            cls._replace_bot(getattr(new_obj, attr_name), memo), memo
+                        ),
+                    )
+                if '__dict__' in obj.__slots__:
+                    # In this case, we have already covered the case that obj has __dict__
+                    # Note that obj may have a __dict__ even if it's not in __slots__!
+                    memo[obj_id] = new_obj
+                    return new_obj
+            if hasattr(obj, '__dict__'):
+                for attr_name, attr in new_obj.__dict__.items():
+                    setattr(new_obj, attr_name, cls._replace_bot(attr, memo))
+                memo[obj_id] = new_obj
+                return new_obj
+        except Exception as exception:
+            warnings.warn(
+                f'Parsing of an object failed with the following exception: {exception}. '
+                f'See the docs of BasePersistence.replace_bot for more information.',
+                RuntimeWarning,
+            )
 
+        memo[obj_id] = obj
         return obj
 
     def insert_bot(self, obj: object) -> object:
@@ -213,7 +312,9 @@ class BasePersistence(ABC):
         Replaces all instances of :attr:`REPLACED_BOT` that occur within the passed object with
         :attr:`bot`. Currently, this handles objects of type ``list``, ``tuple``, ``set``,
         ``frozenset``, ``dict``, ``defaultdict`` and objects that have a ``__dict__`` or
-        ``__slot__`` attribute, excluding objects that can't be copied with `copy.copy`.
+        ``__slots__`` attribute, excluding classes and objects that can't be copied with
+        ``copy.copy``. If the parsing of an object fails, the object will be returned unchanged and
+        the error will be logged.
 
         Args:
             obj (:obj:`object`): The object
@@ -223,7 +324,7 @@ class BasePersistence(ABC):
         """
         return self._insert_bot(obj, {})
 
-    def _insert_bot(self, obj: object, memo: Dict[int, Any]) -> object:  # pylint: disable=R0911
+    def _insert_bot(self, obj: object, memo: Dict[int, object]) -> object:  # pylint: disable=R0911
         obj_id = id(obj)
         if obj_id in memo:
             return memo[obj_id]
@@ -246,6 +347,14 @@ class BasePersistence(ABC):
             new_immutable = obj.__class__(self._insert_bot(item, memo) for item in obj)
             memo[obj_id] = new_immutable
             return new_immutable
+        if isinstance(obj, type):
+            # classes usually do have a __dict__, but it's not writable
+            warnings.warn(
+                'BasePersistence.insert_bot does not handle classes. See '
+                'the docs of BasePersistence.insert_bot for more information.',
+                RuntimeWarning,
+            )
+            return obj
 
         try:
             new_obj = copy(obj)
@@ -270,56 +379,81 @@ class BasePersistence(ABC):
                 new_obj[self._insert_bot(k, memo)] = self._insert_bot(val, memo)
             memo[obj_id] = new_obj
             return new_obj
-        if hasattr(obj, '__dict__'):
-            for attr_name, attr in new_obj.__dict__.items():
-                setattr(new_obj, attr_name, self._insert_bot(attr, memo))
-            memo[obj_id] = new_obj
-            return new_obj
-        if hasattr(obj, '__slots__'):
-            for attr_name in obj.__slots__:
-                setattr(
-                    new_obj,
-                    attr_name,
-                    self._insert_bot(self._insert_bot(getattr(new_obj, attr_name), memo), memo),
-                )
-            memo[obj_id] = new_obj
-            return new_obj
+        try:
+            if hasattr(obj, '__slots__'):
+                for attr_name in obj.__slots__:
+                    setattr(
+                        new_obj,
+                        attr_name,
+                        self._insert_bot(
+                            self._insert_bot(getattr(new_obj, attr_name), memo), memo
+                        ),
+                    )
+                if '__dict__' in obj.__slots__:
+                    # In this case, we have already covered the case that obj has __dict__
+                    # Note that obj may have a __dict__ even if it's not in __slots__!
+                    memo[obj_id] = new_obj
+                    return new_obj
+            if hasattr(obj, '__dict__'):
+                for attr_name, attr in new_obj.__dict__.items():
+                    setattr(new_obj, attr_name, self._insert_bot(attr, memo))
+                memo[obj_id] = new_obj
+                return new_obj
+        except Exception as exception:
+            warnings.warn(
+                f'Parsing of an object failed with the following exception: {exception}. '
+                f'See the docs of BasePersistence.insert_bot for more information.',
+                RuntimeWarning,
+            )
 
+        memo[obj_id] = obj
         return obj
 
     @abstractmethod
-    def get_user_data(self) -> DefaultDict[int, Dict[Any, Any]]:
-        """ "Will be called by :class:`telegram.ext.Dispatcher` upon creation with a
+    def get_user_data(self) -> DefaultDict[int, UD]:
+        """Will be called by :class:`telegram.ext.Dispatcher` upon creation with a
         persistence object. It should return the ``user_data`` if stored, or an empty
-        ``defaultdict(dict)``.
+        :obj:`defaultdict(telegram.ext.utils.types.UD)` with integer keys.
 
         Returns:
-            :obj:`defaultdict`: The restored user data.
+            DefaultDict[:obj:`int`, :class:`telegram.ext.utils.types.UD`]: The restored user data.
         """
 
     @abstractmethod
-    def get_chat_data(self) -> DefaultDict[int, Dict[Any, Any]]:
-        """ "Will be called by :class:`telegram.ext.Dispatcher` upon creation with a
+    def get_chat_data(self) -> DefaultDict[int, CD]:
+        """Will be called by :class:`telegram.ext.Dispatcher` upon creation with a
         persistence object. It should return the ``chat_data`` if stored, or an empty
-        ``defaultdict(dict)``.
+        :obj:`defaultdict(telegram.ext.utils.types.CD)` with integer keys.
 
         Returns:
-            :obj:`defaultdict`: The restored chat data.
+            DefaultDict[:obj:`int`, :class:`telegram.ext.utils.types.CD`]: The restored chat data.
         """
 
     @abstractmethod
-    def get_bot_data(self) -> Dict[Any, Any]:
-        """ "Will be called by :class:`telegram.ext.Dispatcher` upon creation with a
+    def get_bot_data(self) -> BD:
+        """Will be called by :class:`telegram.ext.Dispatcher` upon creation with a
         persistence object. It should return the ``bot_data`` if stored, or an empty
-        :obj:`dict`.
+        :class:`telegram.ext.utils.types.BD`.
 
         Returns:
-            :obj:`dict`: The restored bot data.
+            :class:`telegram.ext.utils.types.BD`: The restored bot data.
         """
 
+    def get_callback_data(self) -> Optional[CDCData]:
+        """Will be called by :class:`telegram.ext.Dispatcher` upon creation with a
+        persistence object. If callback data was stored, it should be returned.
+
+        .. versionadded:: 13.6
+
+        Returns:
+            Optional[:class:`telegram.ext.utils.types.CDCData`]: The restored meta data or
+            :obj:`None`, if no data was stored.
+        """
+        raise NotImplementedError
+
     @abstractmethod
     def get_conversations(self, name: str) -> ConversationDict:
-        """ "Will be called by :class:`telegram.ext.Dispatcher` when a
+        """Will be called by :class:`telegram.ext.Dispatcher` when a
         :class:`telegram.ext.ConversationHandler` is added if
         :attr:`telegram.ext.ConversationHandler.persistent` is :obj:`True`.
         It should return the conversations for the handler with `name` or an empty :obj:`dict`
@@ -335,8 +469,8 @@ class BasePersistence(ABC):
     def update_conversation(
         self, name: str, key: Tuple[int, ...], new_state: Optional[object]
     ) -> None:
-        """Will be called when a :attr:`telegram.ext.ConversationHandler.update_state`
-        is called. This allows the storage of the new state in the persistence.
+        """Will be called when a :class:`telegram.ext.ConversationHandler` changes states.
+        This allows the storage of the new state in the persistence.
 
         Args:
             name (:obj:`str`): The handler's name.
@@ -345,34 +479,84 @@ class BasePersistence(ABC):
         """
 
     @abstractmethod
-    def update_user_data(self, user_id: int, data: Dict) -> None:
+    def update_user_data(self, user_id: int, data: UD) -> None:
         """Will be called by the :class:`telegram.ext.Dispatcher` after a handler has
         handled an update.
 
         Args:
             user_id (:obj:`int`): The user the data might have been changed for.
-            data (:obj:`dict`): The :attr:`telegram.ext.dispatcher.user_data` [user_id].
+            data (:class:`telegram.ext.utils.types.UD`): The
+                :attr:`telegram.ext.Dispatcher.user_data` ``[user_id]``.
         """
 
     @abstractmethod
-    def update_chat_data(self, chat_id: int, data: Dict) -> None:
+    def update_chat_data(self, chat_id: int, data: CD) -> None:
         """Will be called by the :class:`telegram.ext.Dispatcher` after a handler has
         handled an update.
 
         Args:
             chat_id (:obj:`int`): The chat the data might have been changed for.
-            data (:obj:`dict`): The :attr:`telegram.ext.dispatcher.chat_data` [chat_id].
+            data (:class:`telegram.ext.utils.types.CD`): The
+                :attr:`telegram.ext.Dispatcher.chat_data` ``[chat_id]``.
         """
 
     @abstractmethod
-    def update_bot_data(self, data: Dict) -> None:
+    def update_bot_data(self, data: BD) -> None:
         """Will be called by the :class:`telegram.ext.Dispatcher` after a handler has
         handled an update.
 
         Args:
-            data (:obj:`dict`): The :attr:`telegram.ext.dispatcher.bot_data` .
+            data (:class:`telegram.ext.utils.types.BD`): The
+                :attr:`telegram.ext.Dispatcher.bot_data`.
         """
 
+    def refresh_user_data(self, user_id: int, user_data: UD) -> None:
+        """Will be called by the :class:`telegram.ext.Dispatcher` before passing the
+        :attr:`user_data` to a callback. Can be used to update data stored in :attr:`user_data`
+        from an external source.
+
+        .. versionadded:: 13.6
+
+        Args:
+            user_id (:obj:`int`): The user ID this :attr:`user_data` is associated with.
+            user_data (:class:`telegram.ext.utils.types.UD`): The ``user_data`` of a single user.
+        """
+
+    def refresh_chat_data(self, chat_id: int, chat_data: CD) -> None:
+        """Will be called by the :class:`telegram.ext.Dispatcher` before passing the
+        :attr:`chat_data` to a callback. Can be used to update data stored in :attr:`chat_data`
+        from an external source.
+
+        .. versionadded:: 13.6
+
+        Args:
+            chat_id (:obj:`int`): The chat ID this :attr:`chat_data` is associated with.
+            chat_data (:class:`telegram.ext.utils.types.CD`): The ``chat_data`` of a single chat.
+        """
+
+    def refresh_bot_data(self, bot_data: BD) -> None:
+        """Will be called by the :class:`telegram.ext.Dispatcher` before passing the
+        :attr:`bot_data` to a callback. Can be used to update data stored in :attr:`bot_data`
+        from an external source.
+
+        .. versionadded:: 13.6
+
+        Args:
+            bot_data (:class:`telegram.ext.utils.types.BD`): The ``bot_data``.
+        """
+
+    def update_callback_data(self, data: CDCData) -> None:
+        """Will be called by the :class:`telegram.ext.Dispatcher` after a handler has
+        handled an update.
+
+        .. versionadded:: 13.6
+
+        Args:
+            data (:class:`telegram.ext.utils.types.CDCData`): The relevant data to restore
+                :class:`telegram.ext.CallbackDataCache`.
+        """
+        raise NotImplementedError
+
     def flush(self) -> None:
         """Will be called by :class:`telegram.ext.Updater` upon receiving a stop signal. Gives the
         persistence a chance to finish up saving or close a database connection gracefully.
