@@ -17,13 +17,14 @@
 # You should have received a copy of the GNU Lesser Public License
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
 """This module contains the ShippingQueryHandler class."""
-from typing import Any
+
 
 from telegram import Update
 from .handler import Handler
+from .utils.types import CCT
 
 
-class ShippingQueryHandler(Handler[Update]):
+class ShippingQueryHandler(Handler[Update, CCT]):
     """Handler class to handle Telegram shipping callback queries.
 
     Note:
@@ -81,7 +82,9 @@ class ShippingQueryHandler(Handler[Update]):
 
     """
 
-    def check_update(self, update: Any) -> bool:
+    __slots__ = ()
+
+    def check_update(self, update: object) -> bool:
         """Determines whether an update should be passed to this handlers :attr:`callback`.
 
         Args:
