@@ -20,7 +20,8 @@
 from typing import TYPE_CHECKING, Any
 
 from telegram import TelegramObject
-from telegram.utils.types import JSONDict
+from telegram.utils.helpers import DEFAULT_NONE
+from telegram.utils.types import JSONDict, ODVInput
 
 if TYPE_CHECKING:
     from telegram import Bot, File
@@ -64,6 +65,15 @@ class ChatPhoto(TelegramObject):
 
     """
 
+    __slots__ = (
+        'big_file_unique_id',
+        'bot',
+        'small_file_id',
+        'small_file_unique_id',
+        'big_file_id',
+        '_id_attrs',
+    )
+
     def __init__(
         self,
         small_file_id: str,
@@ -85,7 +95,9 @@ class ChatPhoto(TelegramObject):
             self.big_file_unique_id,
         )
 
-    def get_small_file(self, timeout: float = None, api_kwargs: JSONDict = None) -> 'File':
+    def get_small_file(
+        self, timeout: ODVInput[float] = DEFAULT_NONE, api_kwargs: JSONDict = None
+    ) -> 'File':
         """Convenience wrapper over :attr:`telegram.Bot.get_file` for getting the
         small (160x160) chat photo
 
@@ -95,14 +107,16 @@ class ChatPhoto(TelegramObject):
             :class:`telegram.File`
 
         Raises:
-            :class:`telegram.TelegramError`
+            :class:`telegram.error.TelegramError`
 
         """
         return self.bot.get_file(
             file_id=self.small_file_id, timeout=timeout, api_kwargs=api_kwargs
         )
 
-    def get_big_file(self, timeout: float = None, api_kwargs: JSONDict = None) -> 'File':
+    def get_big_file(
+        self, timeout: ODVInput[float] = DEFAULT_NONE, api_kwargs: JSONDict = None
+    ) -> 'File':
         """Convenience wrapper over :attr:`telegram.Bot.get_file` for getting the
         big (640x640) chat photo
 
@@ -112,7 +126,7 @@ class ChatPhoto(TelegramObject):
             :class:`telegram.File`
 
         Raises:
-            :class:`telegram.TelegramError`
+            :class:`telegram.error.TelegramError`
 
         """
         return self.bot.get_file(file_id=self.big_file_id, timeout=timeout, api_kwargs=api_kwargs)
