@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2020
+# Copyright (C) 2015-2021
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -18,36 +18,50 @@
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
 """This module contains an object that represents a Telegram GameHighScore."""
 
+from typing import TYPE_CHECKING, Optional
+
 from telegram import TelegramObject, User
+from telegram.utils.types import JSONDict
+
+if TYPE_CHECKING:
+    from telegram import Bot
 
 
 class GameHighScore(TelegramObject):
     """This object represents one row of the high scores table for a game.
 
-    Attributes:
-        position (:obj:`int`): Position in high score table for the game.
-        user (:class:`telegram.User`): User.
-        score (:obj:`int`): Score.
+    Objects of this class are comparable in terms of equality. Two objects of this class are
+    considered equal, if their :attr:`position`, :attr:`user` and :attr:`score` are equal.
 
     Args:
         position (:obj:`int`): Position in high score table for the game.
         user (:class:`telegram.User`): User.
         score (:obj:`int`): Score.
 
+    Attributes:
+        position (:obj:`int`): Position in high score table for the game.
+        user (:class:`telegram.User`): User.
+        score (:obj:`int`): Score.
+
     """
 
-    def __init__(self, position, user, score):
+    __slots__ = ('position', 'user', 'score', '_id_attrs')
+
+    def __init__(self, position: int, user: User, score: int):
         self.position = position
         self.user = user
         self.score = score
 
+        self._id_attrs = (self.position, self.user, self.score)
+
     @classmethod
-    def de_json(cls, data, bot):
+    def de_json(cls, data: Optional[JSONDict], bot: 'Bot') -> Optional['GameHighScore']:
+        """See :meth:`telegram.TelegramObject.de_json`."""
+        data = cls._parse_data(data)
+
         if not data:
             return None
 
-        data = super().de_json(data, bot)
-
         data['user'] = User.de_json(data.get('user'), bot)
 
         return cls(**data)
