@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2020
+# Copyright (C) 2015-2021
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -18,8 +18,13 @@
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
 """This module contains the classes that represent Telegram InlineQueryResultVenue."""
 
+from typing import TYPE_CHECKING, Any
+
 from telegram import InlineQueryResult
 
+if TYPE_CHECKING:
+    from telegram import InputMessageContent, ReplyMarkup
+
 
 class InlineQueryResultVenue(InlineQueryResult):
     """
@@ -27,24 +32,9 @@ class InlineQueryResultVenue(InlineQueryResult):
     use :attr:`input_message_content` to send a message with the specified content instead of the
     venue.
 
-    Attributes:
-        type (:obj:`str`): 'venue'.
-        id (:obj:`str`): Unique identifier for this result, 1-64 Bytes.
-        latitude (:obj:`float`): Latitude of the venue location in degrees.
-        longitude (:obj:`float`): Longitude of the venue location in degrees.
-        title (:obj:`str`): Title of the venue.
-        address (:obj:`str`): Address of the venue.
-        foursquare_id (:obj:`str`): Optional. Foursquare identifier of the venue if known.
-        foursquare_type (:obj:`str`): Optional. Foursquare type of the venue, if known.
-            (For example, "arts_entertainment/default", "arts_entertainment/aquarium" or
-            "food/icecream".)
-        reply_markup (:class:`telegram.InlineKeyboardMarkup`): Optional. Inline keyboard attached
-            to the message.
-        input_message_content (:class:`telegram.InputMessageContent`): Optional. Content of the
-            message to be sent instead of the venue.
-        thumb_url (:obj:`str`): Optional. Url of the thumbnail for the result.
-        thumb_width (:obj:`int`): Optional. Thumbnail width.
-        thumb_height (:obj:`int`): Optional. Thumbnail height.
+    Note:
+      Foursquare details and Google Pace details are mutually exclusive. However, this
+      behaviour is undocumented and might be changed by Telegram.
 
     Args:
         id (:obj:`str`): Unique identifier for this result, 1-64 Bytes.
@@ -56,6 +46,9 @@ class InlineQueryResultVenue(InlineQueryResult):
         foursquare_type (:obj:`str`, optional): Foursquare type of the venue, if known.
             (For example, "arts_entertainment/default", "arts_entertainment/aquarium" or
             "food/icecream".)
+        google_place_id (:obj:`str`, optional): Google Places identifier of the venue.
+        google_place_type (:obj:`str`, optional): Google Places type of the venue. (See
+            `supported types <https://developers.google.com/places/web-service/supported_types>`_.)
         reply_markup (:class:`telegram.InlineKeyboardMarkup`, optional): Inline keyboard attached
             to the message.
         input_message_content (:class:`telegram.InputMessageContent`, optional): Content of the
@@ -65,22 +58,61 @@ class InlineQueryResultVenue(InlineQueryResult):
         thumb_height (:obj:`int`, optional): Thumbnail height.
         **kwargs (:obj:`dict`): Arbitrary keyword arguments.
 
+    Attributes:
+        type (:obj:`str`): 'venue'.
+        id (:obj:`str`): Unique identifier for this result, 1-64 Bytes.
+        latitude (:obj:`float`): Latitude of the venue location in degrees.
+        longitude (:obj:`float`): Longitude of the venue location in degrees.
+        title (:obj:`str`): Title of the venue.
+        address (:obj:`str`): Address of the venue.
+        foursquare_id (:obj:`str`): Optional. Foursquare identifier of the venue if known.
+        foursquare_type (:obj:`str`): Optional. Foursquare type of the venue, if known.
+        google_place_id (:obj:`str`): Optional. Google Places identifier of the venue.
+        google_place_type (:obj:`str`): Optional. Google Places type of the venue.
+        reply_markup (:class:`telegram.InlineKeyboardMarkup`): Optional. Inline keyboard attached
+            to the message.
+        input_message_content (:class:`telegram.InputMessageContent`): Optional. Content of the
+            message to be sent instead of the venue.
+        thumb_url (:obj:`str`): Optional. Url of the thumbnail for the result.
+        thumb_width (:obj:`int`): Optional. Thumbnail width.
+        thumb_height (:obj:`int`): Optional. Thumbnail height.
+
     """
 
-    def __init__(self,
-                 id,
-                 latitude,
-                 longitude,
-                 title,
-                 address,
-                 foursquare_id=None,
-                 foursquare_type=None,
-                 reply_markup=None,
-                 input_message_content=None,
-                 thumb_url=None,
-                 thumb_width=None,
-                 thumb_height=None,
-                 **kwargs):
+    __slots__ = (
+        'longitude',
+        'reply_markup',
+        'google_place_type',
+        'thumb_width',
+        'thumb_height',
+        'title',
+        'address',
+        'foursquare_id',
+        'foursquare_type',
+        'google_place_id',
+        'input_message_content',
+        'latitude',
+        'thumb_url',
+    )
+
+    def __init__(
+        self,
+        id: str,  # pylint: disable=W0622
+        latitude: float,
+        longitude: float,
+        title: str,
+        address: str,
+        foursquare_id: str = None,
+        foursquare_type: str = None,
+        reply_markup: 'ReplyMarkup' = None,
+        input_message_content: 'InputMessageContent' = None,
+        thumb_url: str = None,
+        thumb_width: int = None,
+        thumb_height: int = None,
+        google_place_id: str = None,
+        google_place_type: str = None,
+        **_kwargs: Any,
+    ):
 
         # Required
         super().__init__('venue', id)
@@ -92,6 +124,8 @@ class InlineQueryResultVenue(InlineQueryResult):
         # Optional
         self.foursquare_id = foursquare_id
         self.foursquare_type = foursquare_type
+        self.google_place_id = google_place_id
+        self.google_place_type = google_place_type
         self.reply_markup = reply_markup
         self.input_message_content = input_message_content
         self.thumb_url = thumb_url
