@@ -25,15 +25,10 @@ class InputMessageContent(TelegramObject):
     """Base class for Telegram InputMessageContent Objects.
 
     See: :class:`telegram.InputContactMessageContent`,
+    :class:`telegram.InputInvoiceMessageContent`,
     :class:`telegram.InputLocationMessageContent`, :class:`telegram.InputTextMessageContent` and
     :class:`telegram.InputVenueMessageContent` for more details.
 
     """
 
-    @property
-    def _has_parse_mode(self) -> bool:
-        return hasattr(self, 'parse_mode')
-
-    @property
-    def _has_disable_web_page_preview(self) -> bool:
-        return hasattr(self, 'disable_web_page_preview')
+    __slots__ = ()
