@@ -25,7 +25,6 @@ import time
 
 from collections import defaultdict
 from html import escape
-from numbers import Number
 from pathlib import Path
 
 from typing import (
@@ -39,15 +38,25 @@ from typing import (
     Type,
     cast,
     IO,
+    TypeVar,
+    Generic,
+    overload,
 )
 
-import pytz  # pylint: disable=E0401
-
 from telegram.utils.types import JSONDict, FileInput
 
 if TYPE_CHECKING:
     from telegram import Message, Update, TelegramObject, InputFile
 
+# in PTB-Raw we don't have pytz, so we make a little workaround here
+DTM_UTC = dtm.timezone.utc
+try:
+    import pytz
+
+    UTC = pytz.utc
+except ImportError:
+    UTC = DTM_UTC  # type: ignore[assignment]
+
 try:
     import ujson as json
 except ImportError:
@@ -114,7 +123,7 @@ def parse_file_input(
 
     Returns:
         :obj:`str` | :class:`telegram.InputFile` | :obj:`object`: The parsed input or the untouched
-            :attr:`file_input`, in case it's no valid file input.
+        :attr:`file_input`, in case it's no valid file input.
     """
     # Importing on file-level yields cyclic Import Errors
     from telegram import InputFile  # pylint: disable=C0415
@@ -176,20 +185,28 @@ def _datetime_to_float_timestamp(dt_obj: dtm.datetime) -> float:
     return dt_obj.timestamp()
 
 
+def _localize(datetime: dtm.datetime, tzinfo: dtm.tzinfo) -> dtm.datetime:
+    """Localize the datetime, where UTC is handled depending on whether pytz is available or not"""
+    if tzinfo is DTM_UTC:
+        return datetime.replace(tzinfo=DTM_UTC)
+    return tzinfo.localize(datetime)  # type: ignore[attr-defined]
+
+
 def to_float_timestamp(
     time_object: Union[int, float, dtm.timedelta, dtm.datetime, dtm.time],
     reference_timestamp: float = None,
-    tzinfo: pytz.BaseTzInfo = None,
+    tzinfo: dtm.tzinfo = None,
 ) -> float:
     """
     Converts a given time object to a float POSIX timestamp.
     Used to convert different time specifications to a common format. The time object
     can be relative (i.e. indicate a time increment, or a time of day) or absolute.
-    Any objects from the :class:`datetime` module that are timezone-naive will be assumed
+    object objects from the :class:`datetime` module that are timezone-naive will be assumed
     to be in UTC, if ``bot`` is not passed or ``bot.defaults`` is :obj:`None`.
 
     Args:
-        time_object (int | float | datetime.timedelta | datetime.datetime | datetime.time):
+        time_object (:obj:`int` | :obj:`float` | :obj:`datetime.timedelta` | \
+            :obj:`datetime.datetime` | :obj:`datetime.time`):
             Time value to convert. The semantics of this parameter will depend on its type:
 
             * :obj:`int` or :obj:`float` will be interpreted as "seconds from ``reference_t``"
@@ -198,21 +215,26 @@ def to_float_timestamp(
             * :obj:`datetime.datetime` will be interpreted as an absolute date/time value
             * :obj:`datetime.time` will be interpreted as a specific time of day
 
-        reference_timestamp (float, optional): POSIX timestamp that indicates the absolute time
-            from which relative calculations are to be performed (e.g. when ``t`` is given as an
-            :obj:`int`, indicating "seconds from ``reference_t``"). Defaults to now (the time at
+        reference_timestamp (:obj:`float`, optional): POSIX timestamp that indicates the absolute
+            time from which relative calculations are to be performed (e.g. when ``t`` is given as
+            an :obj:`int`, indicating "seconds from ``reference_t``"). Defaults to now (the time at
             which this function is called).
 
             If ``t`` is given as an absolute representation of date & time (i.e. a
-            ``datetime.datetime`` object), ``reference_timestamp`` is not relevant and so its
+            :obj:`datetime.datetime` object), ``reference_timestamp`` is not relevant and so its
             value should be :obj:`None`. If this is not the case, a ``ValueError`` will be raised.
-        tzinfo (:obj:`datetime.tzinfo`, optional): If ``t`` is a naive object from the
+        tzinfo (:obj:`pytz.BaseTzInfo`, optional): If ``t`` is a naive object from the
             :class:`datetime` module, it will be interpreted as this timezone. Defaults to
             ``pytz.utc``.
 
+            Note:
+                Only to be used by ``telegram.ext``.
+
+
     Returns:
-        (float | None) The return value depends on the type of argument ``t``. If ``t`` is
-            given as a time increment (i.e. as a obj:`int`, :obj:`float` or
+        :obj:`float` | :obj:`None`:
+            The return value depends on the type of argument ``t``.
+            If ``t`` is given as a time increment (i.e. as a :obj:`int`, :obj:`float` or
             :obj:`datetime.timedelta`), then the return value will be ``reference_t`` + ``t``.
 
             Else if it is given as an absolute date/time value (i.e. a :obj:`datetime.datetime`
@@ -222,9 +244,10 @@ def to_float_timestamp(
             object), the return value is the nearest future occurrence of that time of day.
 
     Raises:
-        TypeError: if `t`'s type is not one of those described above
+        TypeError: If ``t``'s type is not one of those described above.
+        ValueError: If ``t`` is a :obj:`datetime.datetime` and :obj:`reference_timestamp` is not
+            :obj:`None`.
     """
-
     if reference_timestamp is None:
         reference_timestamp = time.time()
     elif isinstance(time_object, dtm.datetime):
@@ -236,7 +259,7 @@ def to_float_timestamp(
         return reference_timestamp + time_object
 
     if tzinfo is None:
-        tzinfo = pytz.utc
+        tzinfo = UTC
 
     if isinstance(time_object, dtm.time):
         reference_dt = dtm.datetime.fromtimestamp(
@@ -247,7 +270,7 @@ def to_float_timestamp(
 
         aware_datetime = dtm.datetime.combine(reference_date, time_object)
         if aware_datetime.tzinfo is None:
-            aware_datetime = tzinfo.localize(aware_datetime)
+            aware_datetime = _localize(aware_datetime, tzinfo)
 
         # if the time of day has passed today, use tomorrow
         if reference_time > aware_datetime.timetz():
@@ -255,10 +278,8 @@ def to_float_timestamp(
         return _datetime_to_float_timestamp(aware_datetime)
     if isinstance(time_object, dtm.datetime):
         if time_object.tzinfo is None:
-            time_object = tzinfo.localize(time_object)
+            time_object = _localize(time_object, tzinfo)
         return _datetime_to_float_timestamp(time_object)
-    if isinstance(time_object, Number):
-        return reference_timestamp + time_object
 
     raise TypeError(f'Unable to convert {type(time_object).__name__} object to timestamp')
 
@@ -266,7 +287,7 @@ def to_float_timestamp(
 def to_timestamp(
     dt_obj: Union[int, float, dtm.timedelta, dtm.datetime, dtm.time, None],
     reference_timestamp: float = None,
-    tzinfo: pytz.BaseTzInfo = None,
+    tzinfo: dtm.tzinfo = None,
 ) -> Optional[int]:
     """
     Wrapper over :func:`to_float_timestamp` which returns an integer (the float value truncated
@@ -281,21 +302,19 @@ def to_timestamp(
     )
 
 
-def from_timestamp(
-    unixtime: Optional[int], tzinfo: dtm.tzinfo = pytz.utc
-) -> Optional[dtm.datetime]:
+def from_timestamp(unixtime: Optional[int], tzinfo: dtm.tzinfo = UTC) -> Optional[dtm.datetime]:
     """
     Converts an (integer) unix timestamp to a timezone aware datetime object.
-    :obj:`None`s are left alone (i.e. ``from_timestamp(None)`` is :obj:`None`).
+    :obj:`None` s are left alone (i.e. ``from_timestamp(None)`` is :obj:`None`).
 
     Args:
-        unixtime (int): Integer POSIX timestamp.
-        tzinfo (:obj:`datetime.tzinfo`, optional): The timezone, the timestamp is to be converted
-            to. Defaults to UTC.
+        unixtime (:obj:`int`): Integer POSIX timestamp.
+        tzinfo (:obj:`datetime.tzinfo`, optional): The timezone to which the timestamp is to be
+            converted to. Defaults to UTC.
 
     Returns:
-        timezone aware equivalent :obj:`datetime.datetime` value if ``timestamp`` is not
-        :obj:`None`; else :obj:`None`
+        Timezone aware equivalent :obj:`datetime.datetime` value if ``unixtime`` is not
+        :obj:`None`; else :obj:`None`.
     """
     if unixtime is None:
         return None
@@ -311,27 +330,27 @@ def from_timestamp(
 def mention_html(user_id: Union[int, str], name: str) -> str:
     """
     Args:
-        user_id (:obj:`int`) The user's id which you want to mention.
-        name (:obj:`str`) The name the mention is showing.
+        user_id (:obj:`int`): The user's id which you want to mention.
+        name (:obj:`str`): The name the mention is showing.
 
     Returns:
-        :obj:`str`: The inline mention for the user as html.
+        :obj:`str`: The inline mention for the user as HTML.
     """
-    return u'<a href="tg://user?id={}">{}</a>'.format(user_id, escape(name))
+    return f'<a href="tg://user?id={user_id}">{escape(name)}</a>'
 
 
 def mention_markdown(user_id: Union[int, str], name: str, version: int = 1) -> str:
     """
     Args:
-        user_id (:obj:`int`) The user's id which you want to mention.
-        name (:obj:`str`) The name the mention is showing.
-        version (:obj:`int` | :obj:`str`): Use to specify the version of telegrams Markdown.
-            Either ``1`` or ``2``. Defaults to ``1``
+        user_id (:obj:`int`): The user's id which you want to mention.
+        name (:obj:`str`): The name the mention is showing.
+        version (:obj:`int` | :obj:`str`): Use to specify the version of Telegram's Markdown.
+            Either ``1`` or ``2``. Defaults to ``1``.
 
     Returns:
-        :obj:`str`: The inline mention for the user as markdown.
+        :obj:`str`: The inline mention for the user as Markdown.
     """
-    return u'[{}](tg://user?id={})'.format(escape_markdown(name, version=version), user_id)
+    return f'[{escape_markdown(name, version=version)}](tg://user?id={user_id})'
 
 
 def effective_message_type(entity: Union['Message', 'Update']) -> Optional[str]:
@@ -340,13 +359,13 @@ def effective_message_type(entity: Union['Message', 'Update']) -> Optional[str]:
     :class:`telegram.Update`.
 
     Args:
-        entity (:obj:`Update` | :obj:`Message`) The ``update`` or ``message`` to extract from
+        entity (:class:`telegram.Update` | :class:`telegram.Message`): The ``update`` or
+            ``message`` to extract from.
 
     Returns:
-        str: One of ``Message.MESSAGE_TYPES``
+        :obj:`str`: One of ``Message.MESSAGE_TYPES``
 
     """
-
     # Importing on file-level yields cyclic Import Errors
     from telegram import Message, Update  # pylint: disable=C0415
 
@@ -412,9 +431,9 @@ def create_deep_linked_url(bot_username: str, payload: str = None, group: bool =
     return f'{base_url}?{key}={payload}'
 
 
-def encode_conversations_to_json(conversations: Dict[str, Dict[Tuple, Any]]) -> str:
+def encode_conversations_to_json(conversations: Dict[str, Dict[Tuple, object]]) -> str:
     """Helper method to encode a conversations dict (that uses tuples as keys) to a
-    JSON-serializable way. Use :attr:`_decode_conversations_from_json` to decode.
+    JSON-serializable way. Use :meth:`decode_conversations_from_json` to decode.
 
     Args:
         conversations (:obj:`dict`): The conversations dict to transform to JSON.
@@ -430,9 +449,9 @@ def encode_conversations_to_json(conversations: Dict[str, Dict[Tuple, Any]]) ->
     return json.dumps(tmp)
 
 
-def decode_conversations_from_json(json_string: str) -> Dict[str, Dict[Tuple, Any]]:
+def decode_conversations_from_json(json_string: str) -> Dict[str, Dict[Tuple, object]]:
     """Helper method to decode a conversations dict (that uses tuples as keys) from a
-    JSON-string created with :attr:`_encode_conversations_to_json`.
+    JSON-string created with :meth:`encode_conversations_to_json`.
 
     Args:
         json_string (:obj:`str`): The conversations dict as JSON string.
@@ -441,7 +460,7 @@ def decode_conversations_from_json(json_string: str) -> Dict[str, Dict[Tuple, An
         :obj:`dict`: The conversations dict after decoding
     """
     tmp = json.loads(json_string)
-    conversations: Dict[str, Dict[Tuple, Any]] = {}
+    conversations: Dict[str, Dict[Tuple, object]] = {}
     for handler, states in tmp.items():
         conversations[handler] = {}
         for key, state in states.items():
@@ -449,7 +468,7 @@ def decode_conversations_from_json(json_string: str) -> Dict[str, Dict[Tuple, An
     return conversations
 
 
-def decode_user_chat_data_from_json(data: str) -> DefaultDict[int, Dict[Any, Any]]:
+def decode_user_chat_data_from_json(data: str) -> DefaultDict[int, Dict[object, object]]:
     """Helper method to decode chat or user data (that uses ints as keys) from a
     JSON-string.
 
@@ -459,8 +478,7 @@ def decode_user_chat_data_from_json(data: str) -> DefaultDict[int, Dict[Any, Any
     Returns:
         :obj:`dict`: The user/chat_data defaultdict after decoding
     """
-
-    tmp: DefaultDict[int, Dict[Any, Any]] = defaultdict(dict)
+    tmp: DefaultDict[int, Dict[object, object]] = defaultdict(dict)
     decoded_data = json.loads(data)
     for user, user_data in decoded_data.items():
         user = int(user)
@@ -474,7 +492,11 @@ def decode_user_chat_data_from_json(data: str) -> DefaultDict[int, Dict[Any, Any
     return tmp
 
 
-class DefaultValue:
+DVType = TypeVar('DVType', bound=object)
+OT = TypeVar('OT', bound=object)
+
+
+class DefaultValue(Generic[DVType]):
     """Wrapper for immutable default arguments that allows to check, if the default value was set
     explicitly. Usage::
 
@@ -485,7 +507,7 @@ class DefaultValue:
                 arg = arg.value
             else:
                 print('`arg` was set explicitly')
-            print('`arg` = ' + str(arg))
+            print(f'`arg` = {str(arg)}')
 
     This yields::
 
@@ -511,6 +533,9 @@ class DefaultValue:
         if value:
             ...
 
+    ``repr(DefaultValue(value))`` returns ``repr(value)`` and ``str(DefaultValue(value))`` returns
+    ``f'DefaultValue({value})'``.
+
     Args:
         value (:obj:`obj`): The value of the default argument
 
@@ -519,15 +544,53 @@ class DefaultValue:
 
     """
 
-    def __init__(self, value: Any = None):
+    __slots__ = ('value', '__dict__')
+
+    def __init__(self, value: DVType = None):
         self.value = value
 
     def __bool__(self) -> bool:
         return bool(self.value)
 
+    @overload
+    @staticmethod
+    def get_value(obj: 'DefaultValue[OT]') -> OT:
+        ...
+
+    @overload
+    @staticmethod
+    def get_value(obj: OT) -> OT:
+        ...
+
+    @staticmethod
+    def get_value(obj: Union[OT, 'DefaultValue[OT]']) -> OT:
+        """
+        Shortcut for::
+
+            return obj.value if isinstance(obj, DefaultValue) else obj
+
+        Args:
+            obj (:obj:`object`): The object to process
+
+        Returns:
+            Same type as input, or the value of the input: The value
+        """
+        return obj.value if isinstance(obj, DefaultValue) else obj  # type: ignore[return-value]
+
+    # This is mostly here for readability during debugging
+    def __str__(self) -> str:
+        return f'DefaultValue({self.value})'
+
+    # This is here to have the default instances nicely rendered in the docs
+    def __repr__(self) -> str:
+        return repr(self.value)
+
 
 DEFAULT_NONE: DefaultValue = DefaultValue(None)
-""":class:`DefaultValue`: Default `None`"""
+""":class:`DefaultValue`: Default :obj:`None`"""
 
 DEFAULT_FALSE: DefaultValue = DefaultValue(False)
-""":class:`DefaultValue`: Default `False`"""
+""":class:`DefaultValue`: Default :obj:`False`"""
+
+DEFAULT_20: DefaultValue = DefaultValue(20)
+""":class:`DefaultValue`: Default :obj:`20`"""
